/*
 *   Copyright (c) 2000 S. Noda (HZG02104@nifty.ne.jp)
 *
 *   Permission to use, copy, modify, and distribute this software
 * and its documentation for any purpose is hereby granted provided
 * that the above copyright notice and this permission notice appear
 * in all copies of the software and related documentation.
 *
 *                          NO WARRANTY
 *
 *   THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY WARRANTIES;
 * WITHOUT EVEN THE IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
 * FOR A PARTICULAR PURPOSE.
 */

#include "stdafx.h"
#include "wmzip.h"
#include "ZipDLL.h"
#include "StatusDlg.h"
#include "PasswordDlg.h"
#include "CommentDlg.h"
#include "ConfigDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

static HINSTANCE hDll;

static ZP_VERSION fpZpVersion;
static ZP_INIT fpZpInit;
static ZP_SETOPTIONS fpZpSetOptions;
static ZP_GETOPTIONS fpZpGetOptions;
static ZP_ARCHIVE fpZpArchive;

static ZIPUSERFUNCTIONS init;
static ZPOPT options;
static ZCL archive;

static CStatusDlg* pDlg;

void CreateStatusDlg()
{
	if (!options.fQuiet)
	{
		pDlg = new CStatusDlg();
		pDlg->Create();
		pDlg->m_ctrlOk.EnableWindow(FALSE);
		pDlg->m_ctrlStatus.EnableWindow(FALSE);
		pDlg->m_bCancel = FALSE;
	}
}

void DestroyStatusDlg()
{
	if (!options.fQuiet)
	{
		pDlg->UpdateData();
		pDlg->m_ctrlStatus.EnableWindow();
		pDlg->m_ctrlOk.EnableWindow();
		pDlg->m_ctrlOk.SetFocus();
		
		if (pDlg->m_bClose)
		{
			pDlg->DestroyWindow();
			pDlg = NULL;
		}
	}
}

void DoEvents()
{
	MSG msg;
	while (PeekMessage(&msg, (HWND) NULL, 0, 0, PM_REMOVE))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
}

int WINAPI ZpPrint(LPSTR buf, unsigned long size)
{
	if (!options.fQuiet)
	{
		pDlg->m_ctrlStatus.SetSel(-1,-1);
		pDlg->m_ctrlStatus.ReplaceSel(buf);
	}
	DoEvents();
	
	return (unsigned int) size;
}

int WINAPI ZpPassword(LPSTR p, int n, LPCSTR m, LPCSTR name)
{
	CPasswordDlg dlg;
	
	dlg.m_stPrompt = m;
	
	int nResponse = dlg.DoModal();
	if (nResponse == IDOK)
		lstrcpy(p, dlg.m_stPassword);
	else if (nResponse == IDCANCEL)
		lstrcpy(p, "\0");
	
	return 0;
}

int WINAPI ZpService(LPCSTR filename, unsigned long size)
{
	TRACE("ZpService: %s (%d)\n", filename, size);
	if (!options.fQuiet)
		return pDlg->m_bCancel;
	return FALSE;
}

int WINAPI ZpComment(LPSTR buf)
{
	CCommentDlg dlg;
	
	dlg.m_stPrompt = "enter new zip file comment: ";
	dlg.m_stComment = buf;
	
	int nResponse = dlg.DoModal();
	
	if (nResponse == IDOK)
		lstrcpy(buf, dlg.m_stComment);
	else if (nResponse == IDCANCEL)
		lstrcpy(buf, "\0");
	
	return TRUE;
}

BOOL ZpLoadLibrary()
{
	hDll = LoadLibrary("zip32.dll");
	
	if (hDll)
	{
		fpZpVersion = (ZP_VERSION) GetProcAddress(hDll, "ZpVersion");
		fpZpInit = (ZP_INIT) GetProcAddress(hDll, "ZpInit");
		fpZpSetOptions = (ZP_SETOPTIONS) GetProcAddress(hDll, "ZpSetOptions");
		fpZpGetOptions = (ZP_GETOPTIONS) GetProcAddress(hDll, "ZpGetOptions");
		fpZpArchive = (ZP_ARCHIVE) GetProcAddress(hDll, "ZpArchive");
		
		if (fpZpVersion && fpZpInit && fpZpSetOptions && fpZpGetOptions && fpZpArchive)
			return TRUE;
	}
	
	return FALSE;
}

BOOL ZpCheckLibrary()
{
	ASSERT(fpZpVersion);
	
	ZpVer ver;
	(*fpZpVersion)(&ver);
	
	if (ver.zip.major == 2 && ver.zip.minor == 3)
		return TRUE;
	
	return FALSE;
}

int ZpInitLibrary()
{
	ASSERT(fpZpInit);
	
	init.print = ZpPrint;
	init.comment = ZpComment;
	init.password = ZpPassword;
	init.ServiceApplication = ZpService;
	
	return (*fpZpInit)(&init);
}

void ZpFreeLibrary()
{
	if (hDll)
		FreeLibrary(hDll);
	
	hDll = NULL;
	
	fpZpVersion = NULL;
	fpZpInit = NULL;
	fpZpSetOptions = NULL;
	fpZpGetOptions = NULL;
	fpZpArchive = NULL;
}

void ZpWriteProfile()
{
	CWinApp* pApp = AfxGetApp();
	
	pApp->WriteProfileString("ZpOpt", "Date", options.Date);
	free(options.Date);
	pApp->WriteProfileString("ZpOpt", "szTempDir", options.szTempDir);
	free(options.szTempDir);
	pApp->WriteProfileInt("ZpOpt", "fTemp", options.fTemp);
	pApp->WriteProfileInt("ZpOpt", "fSuffix", options.fSuffix);
	pApp->WriteProfileInt("ZpOpt", "fEncrypt", options.fEncrypt);
	pApp->WriteProfileInt("ZpOpt", "fSystem", options.fSystem);
	pApp->WriteProfileInt("ZpOpt", "fVolume", options.fVolume);
	pApp->WriteProfileInt("ZpOpt", "fExtra", options.fExtra);
	pApp->WriteProfileInt("ZpOpt", "fNoDirEntries", options.fNoDirEntries);
	pApp->WriteProfileInt("ZpOpt", "fExcludeDate", options.fExcludeDate);
	pApp->WriteProfileInt("ZpOpt", "fIncludeDate", options.fIncludeDate);
	pApp->WriteProfileInt("ZpOpt", "fVerbose", options.fVerbose);
	pApp->WriteProfileInt("ZpOpt", "fQuiet", options.fQuiet);
	pApp->WriteProfileInt("ZpOpt", "fCRLF_LF", options.fCRLF_LF);
	pApp->WriteProfileInt("ZpOpt", "fLF_CRLF", options.fLF_CRLF);
	pApp->WriteProfileInt("ZpOpt", "fJunkDir", options.fJunkDir);
	pApp->WriteProfileInt("ZpOpt", "fGrow", options.fGrow);
	pApp->WriteProfileInt("ZpOpt", "fForce", options.fForce);
	pApp->WriteProfileInt("ZpOpt", "fMove", options.fMove);
	pApp->WriteProfileInt("ZpOpt", "fDeleteEntries", options.fDeleteEntries);
	pApp->WriteProfileInt("ZpOpt", "fUpdate", options.fUpdate);
	pApp->WriteProfileInt("ZpOpt", "fFreshen", options.fFreshen);
	pApp->WriteProfileInt("ZpOpt", "fJunkSFX", options.fJunkSFX);
	pApp->WriteProfileInt("ZpOpt", "fLatestTime", options.fLatestTime);
	pApp->WriteProfileInt("ZpOpt", "fComment", options.fComment);
	pApp->WriteProfileInt("ZpOpt", "fOffsets", options.fOffsets);
	pApp->WriteProfileInt("ZpOpt", "fPrivilege", options.fPrivilege);
	pApp->WriteProfileInt("ZpOpt", "fRecurse", options.fRecurse);
	pApp->WriteProfileInt("ZpOpt", "fRepair", options.fRepair);
	pApp->WriteProfileInt("ZpOpt", "fLevel", options.fLevel - 0x30);
}

void ZpGetProfile()
{
	ASSERT(fpZpGetOptions);
	
	CWinApp* pApp = AfxGetApp();
	
	options = (*fpZpGetOptions)();
	
	options.Date = strdup(pApp->GetProfileString("ZpOpt", "Date", "\0"));
	options.szTempDir = strdup(pApp->GetProfileString("ZpOpt", "szTempDir", "\0"));
	options.fTemp = pApp->GetProfileInt("ZpOpt", "fTemp", FALSE);
	options.fSuffix = pApp->GetProfileInt("ZpOpt", "fSuffix", FALSE);
	options.fEncrypt = pApp->GetProfileInt("ZpOpt", "fEncrypt", TRUE);
	options.fSystem = pApp->GetProfileInt("ZpOpt", "fSystem", TRUE);
	options.fVolume = pApp->GetProfileInt("ZpOpt", "fVolume", FALSE);
	options.fExtra = pApp->GetProfileInt("ZpOpt", "fExtra", TRUE);
	options.fNoDirEntries = pApp->GetProfileInt("ZpOpt", "fNoDirEntries", FALSE);
	options.fExcludeDate = pApp->GetProfileInt("ZpOpt", "fExcludeDate", FALSE);
	options.fIncludeDate = pApp->GetProfileInt("ZpOpt", "fIncludeDate", FALSE);
	options.fVerbose = pApp->GetProfileInt("ZpOpt", "fVerbose", TRUE);
	options.fQuiet = pApp->GetProfileInt("ZpOpt", "fQuiet", FALSE);
	options.fCRLF_LF = pApp->GetProfileInt("ZpOpt", "fCRLF_LF", FALSE);
	options.fLF_CRLF = pApp->GetProfileInt("ZpOpt", "fLF_CRLF", FALSE);
	options.fJunkDir = pApp->GetProfileInt("ZpOpt", "fJunkDir", FALSE);
	options.fGrow = pApp->GetProfileInt("ZpOpt", "fGrow", FALSE);
	options.fForce = pApp->GetProfileInt("ZpOpt", "fForce", FALSE);
	options.fMove = pApp->GetProfileInt("ZpOpt", "fMove", FALSE);
	options.fDeleteEntries = pApp->GetProfileInt("ZpOpt", "fDeleteEntries", FALSE);
	options.fUpdate = pApp->GetProfileInt("ZpOpt", "fUpdate", FALSE);
	options.fFreshen = pApp->GetProfileInt("ZpOpt", "fFreshen", FALSE);
	options.fJunkSFX = pApp->GetProfileInt("ZpOpt", "fJunkSFX", FALSE);
	options.fLatestTime = pApp->GetProfileInt("ZpOpt", "fLatestTime", FALSE);
	options.fComment = pApp->GetProfileInt("ZpOpt", "fComment", TRUE);
	options.fOffsets = pApp->GetProfileInt("ZpOpt", "fOffsets", FALSE);
	options.fPrivilege = pApp->GetProfileInt("ZpOpt", "fPrivilege", FALSE);
	options.fRecurse = pApp->GetProfileInt("ZpOpt", "fRecurse", 1);
	options.fRepair = pApp->GetProfileInt("ZpOpt", "fRepair", 0);
	options.fLevel = pApp->GetProfileInt("ZpOpt", "fLevel", 6) + 0x30;
}

void ZpConfig()
{
	CConfigDlg dlg;
	
	dlg.m_stDate = options.Date;
	dlg.m_stTempDir = options.szTempDir;
	dlg.m_bTemp = options.fTemp;
	dlg.m_bSuffix = options.fSuffix;
	dlg.m_bEncrypt = options.fEncrypt;
	dlg.m_bSystem = options.fSystem;
	dlg.m_bVolume = options.fVolume;
	dlg.m_bExtra = options.fExtra;
	dlg.m_bNoDirEntries = options.fNoDirEntries;
	dlg.m_bExcludeDate = options.fExcludeDate;
	dlg.m_bIncludeDate = options.fIncludeDate;
	dlg.m_bVerbose = options.fVerbose;
	dlg.m_bQuiet = options.fQuiet;
	dlg.m_bCRLF_LF = options.fCRLF_LF;
	dlg.m_bLF_CRLF = options.fLF_CRLF;
	dlg.m_bJunkDir = options.fJunkDir;
	dlg.m_bGrow = options.fGrow;
	dlg.m_bForce = options.fForce;
	dlg.m_bMove = options.fMove;
	dlg.m_bDeleteEntries = options.fDeleteEntries;
	dlg.m_bUpdate = options.fUpdate;
	dlg.m_bFreshen = options.fFreshen;
	dlg.m_bJunkSFX = options.fJunkSFX;
	dlg.m_bLatestTime = options.fLatestTime;
	dlg.m_bComment = options.fComment;
	dlg.m_bOffsets = options.fOffsets;
	dlg.m_bPrivilege = options.fPrivilege;
	dlg.m_bEncryption = options.fEncryption;
	dlg.m_nRecurse = options.fRecurse;
	dlg.m_nRepair = options.fRepair;
	dlg.nLevel = options.fLevel - 0x30;
	
	int nResponse = dlg.DoModal();
	if (nResponse == IDOK)
	{
		free(options.Date);
		options.Date = strdup(dlg.m_stDate);
		free(options.szTempDir);
		options.szTempDir = strdup(dlg.m_stTempDir);
		options.fTemp = dlg.m_bTemp;
		options.fSuffix = dlg.m_bSuffix;
		options.fEncrypt = dlg.m_bEncrypt;
		options.fSystem = dlg.m_bSystem;
		options.fVolume = dlg.m_bVolume;
		options.fExtra = dlg.m_bExtra;
		options.fNoDirEntries = dlg.m_bNoDirEntries;
		options.fExcludeDate = dlg.m_bExcludeDate;
		options.fIncludeDate = dlg.m_bIncludeDate;
		options.fVerbose = dlg.m_bVerbose;
		options.fQuiet = dlg.m_bQuiet;
		options.fCRLF_LF = dlg.m_bCRLF_LF;
		options.fLF_CRLF = dlg.m_bLF_CRLF;
		options.fJunkDir = dlg.m_bJunkDir;
		options.fGrow = dlg.m_bGrow;
		options.fForce = dlg.m_bForce;
		options.fMove = dlg.m_bMove;
		options.fDeleteEntries = dlg.m_bDeleteEntries;
		options.fUpdate = dlg.m_bUpdate;
		options.fFreshen = dlg.m_bFreshen;
		options.fJunkSFX = dlg.m_bJunkSFX;
		options.fLatestTime = dlg.m_bLatestTime;
		options.fComment = dlg.m_bComment;
		options.fOffsets = dlg.m_bOffsets;
		options.fPrivilege = dlg.m_bPrivilege;
		options.fRecurse = dlg.m_nRecurse;
		options.fRepair = dlg.m_nRepair;
		options.fLevel = dlg.nLevel + 0x30;
	}
}

int ZpMain(LPCSTR lpszFileName, HDROP hDropInfo)
{
	ASSERT(fpZpSetOptions);
	ASSERT(fpZpArchive);
	
	int ret, i;
	char path[_MAX_PATH], drive[_MAX_DRIVE], dir[_MAX_DIR], fname[_MAX_FNAME], ext[_MAX_EXT];
	char buf[_MAX_PATH];
	
	archive.argc = DragQueryFile(hDropInfo, -1, NULL, 0);
	archive.lpszZipFN = strdup(lpszFileName);
	archive.FNV = (char **) malloc(sizeof(char *) * archive.argc);
	if (!archive.FNV)
		return -1;
	
	for (i = 0; i < archive.argc; i++)
	{
		DragQueryFile(hDropInfo, i, path, sizeof(path));
		_splitpath(path, drive, dir, fname, ext);
		lstrcpy(buf, fname);
		lstrcat(buf, ext);
		archive.FNV[i] = strdup(buf);
		TRACE("archive.FNV [%d]: %s\n", i, archive.FNV[i]);
	}
	lstrcpy(buf, drive);
	lstrcat(buf, dir);
	options.szRootDir = buf;
	TRACE("options.szRootDir: %s\n", options.szRootDir);
	
	CreateStatusDlg();
	(*fpZpSetOptions)(&options);
	ret = (*fpZpArchive)(archive);
	DestroyStatusDlg();
	TRACE("----------------\n");
	
	for (i = 0; i < archive.argc; i++)
	{
		free(archive.FNV[i]);
	}
	free(archive.FNV);
	free(archive.lpszZipFN);
	
	return ret;
}
